/*
 ---------------------------------------------------------------------------
 Copyright (c) 2002, Dr Brian Gladman <                 >, Worcester, UK.
 All rights reserved.

 LICENSE TERMS

 The free distribution and use of this software in both source and binary
 form is allowed (with or without changes) provided that:

   1. distributions of this source code include the above copyright
      notice, this list of conditions and the following disclaimer;

   2. distributions in binary form include the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other associated materials;

   3. the copyright holder's name is not used to endorse products
      built using this software without specific written permission.

 ALTERNATIVELY, provided that this notice is retained in full, this product
 may be distributed under the terms of the GNU General Public License (GPL),
 in which case the provisions of the GPL apply INSTEAD OF those given above.

 DISCLAIMER

 This software is provided 'as is' with no explicit or implied warranties
 in respect of its properties, including, but not limited to, correctness
 and/or fitness for purpose.
 ---------------------------------------------------------------------------
 Issue Date: 24/01/2003

 Modified on 18/02/2009 to support HMAC-SHA256 by Christian Ghisler

 This is an implementation of RFC2898, which specifies key derivation from
 a password and a salt value.
*/

#include <memory.h>
#include "hmac256.h"

#if defined(__cplusplus)
extern "C"
{
#endif

// test vectors from http://cvs.silcnet.org/source/lib/silccrypt/tests/test_hmacsha256.c
// originally taken from draft hmac-sha-256
#define hashtest

#include <windows.h>

#ifdef hashtest
// Test Case #1: HMAC-SHA-256 with 3-byte input and 32-byte key
const unsigned char key1[] = "\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20\x00";
int key1_len = 32;
const unsigned char data1[] = "abc";
const unsigned char data1_digest[] = "\xa2\x1b\x1f\x5d\x4c\xf4\xf7\x3a\x4d\xd9\x39\x75\x0f\x7a\x06\x6a\x7f\x98\xcc\x13\x1c\xb1\x6a\x66\x92\x75\x90\x21\xcf\xab\x81\x81";

// Test Case #2: HMAC-SHA-256 with 56-byte input and 32-byte key
// key2=key1
const unsigned char data2[] = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq";
const unsigned char data2_digest[] = "\x10\x4f\xdc\x12\x57\x32\x8f\x08\x18\x4b\xa7\x31\x31\xc5\x3c\xae\xe6\x98\xe3\x61\x19\x42\x11\x49\xea\x8c\x71\x24\x56\x69\x7d\x30";

// Test Case #3: HMAC-SHA-256 with 112-byte (multi-block) input and 32-byte key
// key3=key1
const unsigned char data3[] = "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopqabcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq";
const unsigned char data3_digest[] = "\x47\x03\x05\xfc\x7e\x40\xfe\x34\xd3\xee\xb3\xe7\x73\xd9\x5a\xab\x73\xac\xf0\xfd\x06\x04\x47\xa5\xeb\x45\x95\xbf\x33\xa9\xd1\xa3";

// Test Case #5:  HMAC-SHA-256 with 28-byte input and 4-byte key
const unsigned char key5[] = "Jefe";
int key5_len = 4;
unsigned char data5[] = "what do ya want for nothing?";
const unsigned char data5_digest[] = "\x5b\xdc\xc1\x46\xbf\x60\x75\x4e\x6a\x04\x24\x26\x08\x95\x75\xc7\x5a\x00\x3f\x08\x9d\x27\x39\x83\x9d\xec\x58\xb9\x64\xec\x38\x43";

// Test Case #10: HMAC-SHA-256 with 73-byte (multi-block) input and 80-byte key
unsigned char key10[80];
int key10_len = 80;
const unsigned char data10[] = "Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data";
const unsigned char data10_digest[] = "\x63\x55\xac\x22\xe8\x90\xd0\xa3\xc8\x48\x1a\x5c\xa4\x82\x5b\xc8\x84\xd3\xe7\xa1\xff\x98\xa2\xfc\x2a\xc7\xd8\xe0\x64\xc3\xb2\xe6";

int test_hmac()
{
    hmac256_ctx c1[1];
    unsigned char uu[SHA256_DIGEST_SIZE];
	int i;

    hmac_sha256_begin(c1);
	hmac_sha256_key(key1, key1_len, c1);
    hmac_sha256_data(data1, strlen(data1), c1);
	hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c1);
	for (i=0;i<SHA256_DIGEST_SIZE;i++) {
		if (uu[i]!=data1_digest[i])
			return 0;
	}
    hmac_sha256_begin(c1);
	hmac_sha256_key(key1, key1_len, c1);
    hmac_sha256_data(data2, strlen(data2), c1);
	hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c1);
	for (i=0;i<SHA256_DIGEST_SIZE;i++) {
		if (uu[i]!=data2_digest[i])
			return 0;
	}
    hmac_sha256_begin(c1);
	hmac_sha256_key(key1, key1_len, c1);
    hmac_sha256_data(data3, strlen(data3), c1);
	hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c1);
	for (i=0;i<SHA256_DIGEST_SIZE;i++) {
		if (uu[i]!=data3_digest[i])
			return 0;
	}
    hmac_sha256_begin(c1);
	hmac_sha256_key(key5, key5_len, c1);
    hmac_sha256_data(data5, strlen(data5), c1);
	hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c1);
	for (i=0;i<SHA256_DIGEST_SIZE;i++) {
		if (uu[i]!=data5_digest[i])
			return 0;
	}
    memset(key10, '\xaa', key10_len);
	hmac_sha256_begin(c1);
	hmac_sha256_key(key10, key10_len, c1);
    hmac_sha256_data(data10, strlen(data10), c1);
	hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c1);
	for (i=0;i<SHA256_DIGEST_SIZE;i++) {
		if (uu[i]!=data10_digest[i])
			return 0;
	}
	return 1;
}

#endif

void derive_key_sha256(const char pwd[],  /* the PASSWORD     */
               unsigned int pwd_len,        /* and its length   */
               const unsigned char salt[],  /* the SALT and its */
               unsigned int salt_len,       /* length           */
               unsigned int iter,   /* the number of iterations */
               unsigned char key[], /* space for the output key */
               unsigned int key_len)/* and its required length  */
{
    unsigned int    i, j, k, n_blk;
    unsigned char uu[SHA256_DIGEST_SIZE], ux[SHA256_DIGEST_SIZE];
    hmac256_ctx c1[1], c2[1], c3[1];

#ifdef hashtest
	if (!test_hmac()) {
		MessageBox(GetActiveWindow(),"HMAC-SHA256 faulty!","test",MB_ICONSTOP);
		return;
	}
#endif


    /* set HMAC context (c1) for password               */
    hmac_sha256_begin(c1);
    hmac_sha256_key(pwd, pwd_len, c1);

    /* set HMAC context (c2) for password and salt      */
    memcpy(c2, c1, sizeof(hmac256_ctx));
    hmac_sha256_data(salt, salt_len, c2);

    /* find the number of SHA blocks in the key         */
    n_blk = 1 + (key_len - 1) / SHA256_DIGEST_SIZE;

    for(i = 0; i < n_blk; ++i) /* for each block in key */
    {
        /* ux[] holds the running xor value             */
        memset(ux, 0, SHA256_DIGEST_SIZE);

        /* set HMAC context (c3) for password and salt  */
        memcpy(c3, c2, sizeof(hmac256_ctx));

        /* enter additional data for 1st block into uu  */
        uu[0] = (unsigned char)((i + 1) >> 24);
        uu[1] = (unsigned char)((i + 1) >> 16);
        uu[2] = (unsigned char)((i + 1) >> 8);
        uu[3] = (unsigned char)(i + 1);

        /* this is the key mixing iteration         */
        for(j = 0, k = 4; j < iter; ++j)
        {
            /* add previous round data to HMAC      */
            hmac_sha256_data(uu, k, c3);

            /* obtain HMAC for uu[]                 */
            hmac_sha256_end(uu, SHA256_DIGEST_SIZE, c3);

            /* xor into the running xor block       */
            for(k = 0; k < SHA256_DIGEST_SIZE; ++k)
                ux[k] ^= uu[k];

            /* set HMAC context (c3) for password   */
            memcpy(c3, c1, sizeof(hmac256_ctx));
        }

        /* compile key blocks into the key output   */
        j = 0; k = i * SHA256_DIGEST_SIZE;
        while(j < SHA256_DIGEST_SIZE && k < key_len)
            key[k++] = ux[j++];
    }
}

#ifdef TEST

#include <stdio.h>

struct
{   unsigned int    pwd_len;
    unsigned int    salt_len;
    unsigned int    it_count;
    unsigned char   *pwd;
    unsigned char   salt[32];
    unsigned char   key[32];
} tests[] =
{
    {   8, 4, 5, (unsigned char*)"password",
        {   0x12, 0x34, 0x56, 0x78 },
        {   0x5c, 0x75, 0xce, 0xf0, 0x1a, 0x96, 0x0d, 0xf7,
            0x4c, 0xb6, 0xb4, 0x9b, 0x9e, 0x38, 0xe6, 0xb5 } /* ... */
    },
    {   8, 8, 5, (unsigned char*)"password",
        {   0x12, 0x34, 0x56, 0x78, 0x78, 0x56, 0x34, 0x12 },
        {   0xd1, 0xda, 0xa7, 0x86, 0x15, 0xf2, 0x87, 0xe6,
            0xa1, 0xc8, 0xb1, 0x20, 0xd7, 0x06, 0x2a, 0x49 } /* ... */
    }
};

int main()
{   unsigned int    i, j, key_len = 256;
    unsigned char   key[256];

    printf("\nTest of RFC2898 Password Based Key Derivation");
    for(i = 0; i < 2; ++i)
    {
        derive_key(tests[i].pwd, tests[i].pwd_len, tests[i].salt,
                    tests[i].salt_len, tests[i].it_count, key, key_len);

        printf("\ntest %i: ", i + 1);
        printf("key %s", memcmp(tests[i].key, key, 16) ? "is bad" : "is good");
        for(j = 0; j < key_len && j < 64; j += 4)
        {
            if(j % 16 == 0)
                printf("\n");
            printf("0x%02x%02x%02x%02x ", key[j], key[j + 1], key[j + 2], key[j + 3]);
        }
        printf(j < key_len ? " ... \n" : "\n");
    }
    printf("\n");
    return 0;
}

#if defined(__cplusplus)
}
#endif

#endif
