#define _WIN32_WINNT 0x400   // for wincrypt!

#if defined(__cplusplus)
extern "C"
{
#endif

#include <windows.h>
#include <wincrypt.h>
#include "passcrypt.h"
#include "zipaes/fileenc.h"
#include "derivekey256.h"
#include "mycrypt.h"

unsigned char initializationvector[16]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};

void CreateAESKeyN256(char* salt,char* masterpass,TAESKey256 *protkey,TAESKey256 *Key)
{
	derive_key_sha256((const char*)masterpass,strlen(masterpass),
		       (const unsigned char*)salt,32, 3000,
		       (unsigned char*)Key, sizeof(TAESKey256));
}

int __stdcall InitPassCryptAES(
    int mode,                               /* the mode to be used (input)          */
    const char pwd[],                       /* the user specified password (input)  */
    unsigned int pwd_len,                   /* the length of the password (input)   */
    const unsigned char salt[],             /* the salt (input)                     */
    char pwd_ver[PWD_VER_LENGTH]         ,  /* 2 byte password verifier (output)    */
    fcrypt_ctx      cx[1])                  /* the file encryption context (output) */
{   unsigned char kbuf[2 * MAX_KEY_LENGTH + PWD_VER_LENGTH];

    if(pwd_len > MAX_PWD_LENGTH)
        return PASSWORD_TOO_LONG;

    cx->mode = mode;   // TC uses 3 = 256 bit
    cx->pwd_len = pwd_len;

    /* derive the encryption and authetication keys and the password verifier   */
	derive_key_sha256(pwd, pwd_len, salt,32,
		       3000,kbuf, 2 * KEY_LENGTH(mode) + PWD_VER_LENGTH);

    /* initialise the encryption nonce and buffer pos   */
    cx->encr_pos = BLOCK_SIZE;
    /* if we need a random component in the encryption  */
    /* nonce, this is where it would have to be set     */
    memset(cx->nonce, 0, BLOCK_SIZE * sizeof(unsigned char));

    /* initialise for encryption using key 1            */
    aes_set_encrypt_key(kbuf, KEY_LENGTH(mode), cx->encr_ctx);

    /* initialise for authentication using key 2        */
    hmac_sha1_begin(cx->auth_ctx);
    hmac_sha1_key(kbuf + KEY_LENGTH(mode), KEY_LENGTH(mode), cx->auth_ctx);
    memcpy(pwd_ver, kbuf + 2 * KEY_LENGTH(mode), PWD_VER_LENGTH);
   
	return GOOD_RETURN;
}

// Return value: 0=verified, 1=unverified, -1=error
int DecryptPassword(char* masterpass,char* salt,char* cryptpass,int cryptlen)
{
	char mac[MAX_KEY_LENGTH];
	char pwd_ver[2];
	fcrypt_ctx Key;
	int retval=-1;

    if (InitPassCryptAES(3,masterpass,strlen(masterpass),(unsigned char*)salt,pwd_ver,&Key)==0) {
		fcrypt_decrypt((unsigned char*)cryptpass,cryptlen-10,&Key);  // Last 10 bytes are the mac!
		if (fcrypt_end((unsigned char*)&mac,&Key)) {
	        cryptlen-=10;
			retval=0;
			for (int i=0;i<10;i++)
				if (cryptpass[cryptlen+i]!=mac[i])
				retval=1;
		}
	}
	return retval;
}

int EncryptPassword(char* masterpass,char* salt,char* cryptpass,int cryptlen,int buflen)
{
	char mac[MAX_KEY_LENGTH];
	char pwd_ver[2];
	fcrypt_ctx Key;

    if (InitPassCryptAES(3,masterpass,strlen(masterpass),(unsigned char*)salt,pwd_ver,&Key)==0) {
		fcrypt_encrypt((unsigned char*)cryptpass,cryptlen,&Key);  // Last 10 bytes are the password verifier!
		if (fcrypt_end((unsigned char*)&mac,&Key)) {
			if (cryptlen+10>=buflen)
				return -1;
			for (int i=0;i<10;i++)
				cryptpass[cryptlen+i]=mac[i];
	        cryptlen+=10;
			return cryptlen;
		}
	}
	return -1;
}

BOOL GenerateRandomByteBlock(char* block,int len)
{
	BOOL result=false;
	HCRYPTPROV hCryptProv;
	if (CryptAcquireContext(&hCryptProv,NULL,NULL,PROV_RSA_FULL,CRYPT_VERIFYCONTEXT)) {
		result=CryptGenRandom(hCryptProv,len,(unsigned char*)block);
		CryptReleaseContext(hCryptProv,0);
	}
	return result;
}

#if defined(__cplusplus)
}
#endif

